@@ -49,21 +49,17 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <mqtt3.h>
 #include <mqtt3_protocol.h>
 #include <memory_mosq.h>
+#include <net_mosq.h>
 
 static uint64_t bytes_received = 0;
 static uint64_t bytes_sent = 0;
 static unsigned long msgs_received = 0;
 static unsigned long msgs_sent = 0;
-static int max_connections = -1;
 
-void mqtt3_net_set_max_connections(int max)
-{
-	max_connections = max;
-}
-
-int mqtt3_socket_accept(mqtt3_context ***contexts, int *context_count, int listensock)
+int mqtt3_socket_accept(struct _mosquitto_db *db, int listensock)
 {
 	int i;
+	int j;
 	int new_sock = -1;
 	mqtt3_context **tmp_contexts = NULL;
 	mqtt3_context *new_context;
@@ -75,14 +71,6 @@ int mqtt3_socket_accept(mqtt3_context ***contexts, int *context_count, int liste
 	new_sock = accept(listensock, NULL, 0);
 	if(new_sock < 0) return -1;
 
-	if(max_connections > 0 && (*context_count) >= max_connections){
-#ifndef WIN32
-		close(new_sock);
-#else
-		closesocket(new_sock);
-#endif
-		return -1;
-	}
 #ifndef WIN32
 	/* Set non-blocking */
 	opt = fcntl(new_sock, F_GETFL, 0);
@@ -105,69 +93,61 @@ int mqtt3_socket_accept(mqtt3_context ***contexts, int *context_count, int liste
 	if(!hosts_access(&wrap_req)){
 		/* Access is denied */
 		mqtt3_log_printf(MOSQ_LOG_NOTICE, "Client connection denied access by tcpd.");
-#ifndef WIN32
-		close(new_sock);
-#else
-		closesocket(new_sock);
-#endif
+		COMPAT_CLOSE(new_sock);
 		return -1;
 	}else{
 #endif
 		new_context = mqtt3_context_init(new_sock);
 		if(!new_context){
-#ifndef WIN32
-			close(new_sock);
-#else
-			closesocket(new_sock);
-#endif
+			COMPAT_CLOSE(new_sock);
+			return -1;
+		}
+		for(i=0; i<db->config->listener_count; i++){
+			for(j=0; j<db->config->listeners[i].sock_count; j++){
+				if(db->config->listeners[i].socks[j] == listensock){
+					new_context->listener = &db->config->listeners[i];
+					break;
+				}
+			}
+		}
+		if(!new_context->listener){
+			COMPAT_CLOSE(new_sock);
+			return -1;
+		}
+
+		if(new_context->listener->max_connections > 0 && new_context->listener->client_count >= new_context->listener->max_connections){
+			COMPAT_CLOSE(new_sock);
 			return -1;
 		}
 		mqtt3_log_printf(MOSQ_LOG_NOTICE, "New client connected from %s.", new_context->core.address);
-		for(i=0; i<(*context_count); i++){
-			if((*contexts)[i] == NULL){
-				(*contexts)[i] = new_context;
+		for(i=0; i<db->context_count; i++){
+			if(db->contexts[i] == NULL){
+				db->contexts[i] = new_context;
 				break;
 			}
 		}
-		if(i==(*context_count)){
-			tmp_contexts = _mosquitto_realloc(*contexts, sizeof(mqtt3_context*)*((*context_count)+1));
+		if(i==db->context_count){
+			tmp_contexts = _mosquitto_realloc(db->contexts, sizeof(mqtt3_context*)*(db->context_count+1));
 			if(tmp_contexts){
-				(*context_count)++;
-				*contexts = tmp_contexts;
-				(*contexts)[(*context_count)-1] = new_context;
+				db->context_count++;
+				db->contexts = tmp_contexts;
+				db->contexts[db->context_count-1] = new_context;
 			}else{
 				mqtt3_context_cleanup(NULL, new_context, true);
 			}
 		}
+		new_context->listener->client_count++;
 #ifdef WITH_WRAP
 	}
 #endif
 	return new_sock;
 }
 
-/* Close a socket associated with a context and set it to -1.
- */
-void mqtt3_socket_close(mqtt3_context *context)
-{
-	int rc = 0;
-
-	assert(context);
-
-	if(context->core.sock != -1){
-#ifndef WIN32
-		rc = close(context->core.sock);
-#else
-		rc = closesocket(context->core.sock);
-#endif
-		context->core.sock = -1;
-	}
-}
-
 /* Creates a socket and listens on port 'port'.
  * Returns 1 on failure
  * Returns 0 on success.
  */
-int mqtt3_socket_listen(const char *host, uint16_t port, int **socks, int *sock_count)
+int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 {
 	int sock = -1;
 	struct addrinfo hints;
@@ -180,16 +160,18 @@ int mqtt3_socket_listen(const char *host, uint16_t port, int **socks, int *sock_
 	char ss_opt = 1;
 #endif
 
-	snprintf(service, 10, "%d", port);
+	if(!listener) return MOSQ_ERR_INVAL;
+
+	snprintf(service, 10, "%d", listener->port);
 	memset(&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = PF_UNSPEC;
 	hints.ai_flags = AI_PASSIVE;
 	hints.ai_socktype = SOCK_STREAM;
 
-	if(getaddrinfo(host, service, &hints, &ainfo)) return INVALID_SOCKET;
+	if(getaddrinfo(listener->host, service, &hints, &ainfo)) return INVALID_SOCKET;
 
-	*sock_count = 0;
-	*socks = NULL;
+	listener->sock_count = 0;
+	listener->socks = NULL;
 
 	for(rp = ainfo; rp; rp = rp->ai_next){
 		if(rp->ai_family == AF_INET){
@@ -205,13 +187,13 @@ int mqtt3_socket_listen(const char *host, uint16_t port, int **socks, int *sock_
 			mqtt3_log_printf(MOSQ_LOG_WARNING, "Warning: %s", strerror(errno));
 			continue;
 		}
-		(*sock_count)++;
-		*socks = _mosquitto_realloc(*socks, sizeof(int)*(*sock_count));
-		if(!(*socks)){
+		listener->sock_count++;
+		listener->socks = _mosquitto_realloc(listener->socks, sizeof(int)*listener->sock_count);
+		if(!listener->socks){
 			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: Out of memory.");
 			return MOSQ_ERR_NOMEM;
 		}
-		(*socks)[(*sock_count)-1] = sock;
+		listener->socks[listener->sock_count-1] = sock;
 
 		ss_opt = 1;
 		setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &ss_opt, sizeof(ss_opt));
@@ -224,51 +206,47 @@ int mqtt3_socket_listen(const char *host, uint16_t port, int **socks, int *sock_
 		opt = fcntl(sock, F_GETFL, 0);
 		if(opt == -1 || fcntl(sock, F_SETFL, opt | O_NONBLOCK) == -1){
 			/* If either fcntl fails, don't want to allow this client to connect. */
-			close(sock);
+			COMPAT_CLOSE(sock);
 			return 1;
 		}
 #else
 		if(ioctlsocket(sock, FIONBIO, &opt)){
-			closesocket(sock);
+			COMPAT_CLOSE(sock);
 			return 1;
 		}
 #endif
 
 		if(bind(sock, rp->ai_addr, rp->ai_addrlen) == -1){
 			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s", strerror(errno));
-#ifndef WIN32
-			close(sock);
-#else
-			closesocket(sock);
-#endif
+			COMPAT_CLOSE(sock);
 			return 1;
 		}
 
 		if(listen(sock, 100) == -1){
 			mqtt3_log_printf(MOSQ_LOG_ERR, "Error: %s", strerror(errno));
-#ifndef WIN32
-			close(sock);
-#else
-			closesocket(sock);
-#endif
+			COMPAT_CLOSE(sock);
 			return 1;
 		}
 	}
 	freeaddrinfo(ainfo);
 
 	/* We need to have at least one working socket. */
-	if(*sock_count > 0){
+	if(listener->sock_count > 0){
 		return 0;
 	}else{
 		return 1;
 	}
 }
 
-int mqtt3_net_read(mosquitto_db *db, mqtt3_context *context)
+int mqtt3_net_read(mosquitto_db *db, int context_index)
 {
 	uint8_t byte;
 	ssize_t read_length;
 	int rc = 0;
+	mqtt3_context *context;
+
+	if(context_index < 0 || context_index >= db->context_count) return MOSQ_ERR_INVAL;
+	context = db->contexts[context_index];
 
 	if(!context || context->core.sock == -1) return MOSQ_ERR_INVAL;
 	/* This gets called if pselect() indicates that there is network data
@@ -365,7 +343,7 @@ int mqtt3_net_read(mosquitto_db *db, mqtt3_context *context)
 	msgs_received++;
 	/* All data for this packet is read. */
 	context->core.in_packet.pos = 0;
-	rc = mqtt3_packet_handle(db, context);
+	rc = mqtt3_packet_handle(db, context_index);
 
 	/* Free data and reset values */
 	_mosquitto_packet_cleanup(&context->core.in_packet);
